/* ------------------------------------------------------------------

   libofa -- the Open Fingerprint Architecture library

   Public Domain (PD) 2006 Predixis Corporation
   No rights reserved.

-------------------------------------------------------------------*/
#include "ofa1/ofa.h"
#ifdef WIN32
#include <io.h>
#endif
#include <fcntl.h>
#include <stdio.h>
#include "protocol.h"

bool readBytes(int fd, unsigned char *buf, int size) {
    int ct = 0;
    while (ct < size) {
	unsigned char tmp[4096];
 
        int x = size - ct;
        if (x > 4096)
	    x = 4096;

        int n = read(fd, tmp, x);

	if (n <= 0) {
            return false;
	}

        for (int i = 0; i < n; ++i) {
            buf[ct + i] = tmp[i];
        }
        ct += n;
    }
    return true;
}

// This method only supports PCM/uncompressed format, with a single fmt
// chunk followed by a single data chunk
bool loadWave(char *file, AudioData *data) 
{
    unsigned char *samples = 0;
    int srate = 0;
    int channels = 0;
    
    int fd = open(file, O_RDONLY | 0x8000);
    if (fd == -1) 
        return false;
    
    if (lseek(fd, 0L, SEEK_SET) == -1L) {
	close(fd);
        return false;
    }
    
    unsigned char hdr[36];
    if (!readBytes(fd, hdr, 36)) {
	close(fd);
        return false;
    }
    if (hdr[0] != 'R' || hdr[1] != 'I' || hdr[2] != 'F' || hdr[3] != 'F') {
	close(fd);
        return false;
    }
    // Note: bytes 4 thru 7 contain the file size - 8 bytes
    if (hdr[8] != 'W' || hdr[9] != 'A' || hdr[10] != 'V' || hdr[11] != 'E') {
	close(fd);
        return false;
    }
    if (hdr[12] != 'f' || hdr[13] != 'm' || hdr[14] != 't' || hdr[15] != ' ') {
	close(fd);
        return false;
    }
    
    long extraBytes = hdr[16] + (hdr[17] << 8) + (hdr[18] << 16) + (hdr[19] << 24) - 16;
    int compression = hdr[20] + (hdr[21] << 8);
    // Type 1 is PCM/Uncompressed
    if (compression != 1) {
	close(fd);
	return false;
    }
    channels = hdr[22] + (hdr[23] << 8);
    // Only mono or stereo PCM is supported in this example
    if (channels < 1 || channels > 2) {
	close(fd);
	return false;
    }
    // Samples per second, independent of number of channels
    srate = hdr[24] + (hdr[25] << 8) + (hdr[26] << 16) + (hdr[27] << 24);
    // Bytes 28-31 contain the "average bytes per second", unneeded here
    // Bytes 32-33 contain the number of bytes per sample (includes channels)
    // Bytes 34-35 contain the number of bits per single sample
    int bits = hdr[34] + (hdr[35] << 8);
    // Supporting othe sample depths will require conversion
    if (bits != 16) {
	close(fd);
	return false;
    }
    
    // Skip past extra bytes, if any
    if (lseek(fd, 36L + extraBytes, SEEK_SET) == -1L) {
	close(fd);
        return false;
    }
    
    // Start reading the next frame.  Only supported frame is the data block
    unsigned char b[8];
    if (!readBytes(fd, b, 8)) {
	close(fd);
        return false;
    }
    
    if (b[0] != 'd' || b[1] != 'a' || b[2] != 't' || b[3] != 'a') {
	close(fd);
	return false;
    }
    int bytes = b[4] + (b[5] << 8) + (b[6] << 16) + (b[7] << 24);

    // NYI: No need to read the whole file, just the first 130 seconds
    // (But we do need to pass the correct total length in to the protocol!)
    samples = new unsigned char[bytes];
    if (!readBytes(fd, samples, bytes)) {
	delete[] samples;
	close(fd);
	return false;
    }

    close(fd);
    data->setData(samples, OFA_LITTLE_ENDIAN, bytes/2, srate, channels == 2 ? 1 : 0);
    return true;
}

int main(int argc, char **argv) {
    for (int i = 1; i < argc; ++i) {
	char *file = argv[i];
	printf("Checking file %s\n", file);
        AudioData *data = new AudioData("client_key", "client_version");
	if (loadWave(file, data)) {
	    if (!data->createPrint()) {
		printf("** Failed to generate print.\n");
	    } else {
                //printf("%s\n", data->info.getPrint().c_str());
                data->info.setFormat("wav");
                int ms = data->getSize()/(data->getSRate()/1000);
		if (data->getStereo())
		    ms /= 2;
                data->info.setLengthInMS(ms);
                data->info.setMetadataFlag(true);
                retrieve_metadata(&data->info);
            }
	} else {
	    printf("** Unable to decode file.\n");
	}
        delete data;
    }
    return 0;
}

